(* @LICENSE@ *)
program LoadClient;

{$APPTYPE CONSOLE}

uses
  BTCommAdapterIndy,
  BTJMSConnection,
  BTJMSConnectionFactory,
  BTJMSInterfaces,
  SysUtils, Classes;

var
  ReceiveDir: string;
  Connection: IConnection;
  Session: ISession;
  Destination: IDestination;
  Consumer: IMessageConsumer;
  BytesMessage: IBytesMessage;
  LocalFileName: TFileName;

  procedure Connect;
  var
    Servername: string;
  begin
    if ParamCount > 0 then
      Servername := ParamStr(1)
    else
      Servername := BTJMSConnectionFactory.DEFAULT_BROKER_URL;

    Connection := TBTJMSConnection.MakeConnection('', '', Servername);
    Connection.Start;
    WriteLn('Connected with ' + Servername);

    Session := Connection.CreateSession(False, amClientAcknowledge);
    Destination := Session.CreateQueue('loadserver');
    Consumer := Session.CreateConsumer(Destination);
  end;

  procedure SaveFile(FileName: TFileName);
  var
    FileStream: TFileStream;
    S: TStringStream;
  begin
    S := TStringStream.Create(BytesMessage.Content);
    try
      FileStream := TFileStream.Create(FileName, fmCreate
        or fmShareDenyWrite);
      try
        FileStream.CopyFrom(S, S.Size);
      finally
        FileStream.Free;
      end
    finally
      S.Free;
    end;
  end;

begin
  ReceiveDir := 'receive.' + FormatDateTime('hhnnsszzz', Now);

  {$I+}
  try
    if not DirectoryExists(ReceiveDir) then
      MkDir(ReceiveDir);
  except
    on E:Exception do
    begin
      WriteLn('Could not create output directory');
      Halt(1);
    end;
  end;

  Connect;
  try
    WriteLn('Listening for new files');
    while True do
    begin
      BytesMessage := Consumer.Receive(1000) as IBytesMessage;
      if Assigned(BytesMessage) then
      begin
        WriteLn('Received message');
        WriteLn('filename=' + BytesMessage.Properties.Item['filename']);
        WriteLn('filesize=' + BytesMessage.Properties.Item['filesize']);
        WriteLn('sequence=' + BytesMessage.Properties.Item['sequence']);
        // save the file
        LocalFileName := ReceiveDir + '\'
          + BytesMessage.Properties.Item['filename']
          + '.'
          + FormatDateTime('yyyymmddhhnnsszzz', Now);
        WriteLn('Local file: ' + LocalFileName);
        try
          try
            SaveFile(LocalFileName);
          except
            on E:Exception do
            begin
              WriteLn(E.Message);
            end;
          end;
        finally
          BytesMessage.Acknowledge;
        end;
      end;
    end;
  finally
    Connection.Close;
  end;
end.
