(* @LICENSE@ *)
program LoadServer;

{$APPTYPE CONSOLE}

uses
  BTCommAdapterIndy,
  BTJMSConnection,
  BTJMSConnectionFactory,
  BTJMSInterfaces,
  JclFileUtils,
  SysUtils, Classes;

const
  SendDir = 'files';
    
var
  Files: TStrings;
  I: Integer;
  Sequence: Integer;
  Connection: IConnection;
  Session: ISession;
  Destination: IDestination;
  Producer: IMessageProducer;

  procedure Connect;
  var
    Servername: string;
  begin
    if ParamCount > 0 then
      Servername := ParamStr(1)
    else
      Servername := BTJMSConnectionFactory.DEFAULT_BROKER_URL;

    Connection := TBTJMSConnection.MakeConnection('', '', Servername);
    Connection.Start;
    WriteLn('Connected with ' + Servername); 
    Session := Connection.CreateSession(False, amAutoAcknowledge);
    Destination := Session.CreateQueue('loadserver');
    Producer := Session.CreateProducer(Destination);
  end;

  procedure Disconnect;
  begin
    Connection.Close;
  end;

  procedure SendFile(const FileName: TFilename);
  var
    FileStream: TFileStream;
    BytesMessage: IBytesMessage;
  begin
    FileStream := TFileStream.Create(Format('%s\%s', [SendDir, FileName]),
      fmOpenRead or fmShareDenyWrite);
    try
      Inc(Sequence);
      try
        BytesMessage := Session.CreateBytesMessage;
        // RawByteString
        BytesMessage.Content := (StringOfChar(' ', FileStream.Size));
        FileStream.Read(PAnsiChar(BytesMessage.Content)^, FileStream.Size);

        Assert(FileStream.Size = Length(BytesMessage.Content));

        BytesMessage.Properties.Item['filename'] := FileName;
        BytesMessage.Properties.Item['filesize'] := IntToStr(FileStream.Size);
        BytesMessage.Properties.Item['sequence'] := IntToStr(Sequence);

        WriteLn('Size: ' + BytesMessage.Properties.Item['filesize']);

        Producer.Send(BytesMessage);
      except
        on E:Exception do
        begin
          WriteLn(E.Message);
        end;
      end;
    finally
      FileStream.Free;
    end;
    // DeleteFile(FileName);

  end;

begin
  {$I+}
  try
    if not DirectoryExists(SendDir) then
     MkDir(SendDir);
  except
    on E:Exception do
    begin
      WriteLn('Could not create files directory');
      Halt(1);
    end;
  end;

  Sequence := 0;

  Files := TStringList.Create;
  try
    while True do
    begin
      WriteLn(Format('Looking for new files in folder "%s"', [SendDir]));
      Files.Clear;
      BuildFileList(Format('%s\*.*', [SendDir]), faAnyFile, Files);
      if Files.Count > 0 then
      begin
        WriteLn(Format('Found %d files', [Files.Count]));
        Connect;
        for I := 0 to Files.Count - 1 do
        begin
          WriteLn('Processing ' + Files[I]);
          SendFile(Files[I]);
          WriteLn('Message sent.');
        end;
        Disconnect;
      end;
      WriteLn('Waiting ...');
      Sleep(5000);
    end;
  finally
    Files.Free;
  end;
end.
