unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, TntStdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    Panel2: TPanel;
    ButtonSend: TButton;
    Panel3: TPanel;
    Label2: TLabel;
    Panel4: TPanel;
    ButtonReceive: TButton;
    procedure ButtonSendClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ButtonReceiveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
    MemoLog: TTntMemo;
    MemoIncoming: TTntMemo;
    TntComboBox1: TTntComboBox;
    procedure Log(const Msg: WideString);
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

uses
  BTJMSInterfaces,
  BTJMSConnection;

{$R *.dfm}

const
  am = '&#x12A0;&#x121B;&#x122D;&#x129B;';
  ar = '&#x0627;&#x0644;&#x0639;&#x0631;&#x0628;&#x064A;&#x0629;';
  gu = '&#x0A97;&#x0AC1;&#x0A9C;&#x0AB0;&#x0ABE;&#x0AA4;&#x0AC0;';
  he = '&#x05E2;&#x05D1;&#x05E8;&#x05D9;&#x05EA;';
  hi = '&#x0939;&#x093F;&#x0928;&#x094D;&#x0926;&#x0940;';
  kn = '&#x0C95;&#x0CA8;&#x0CCD;&#x0CA8;&#x0CA1;';
  mr = '&#x092E;&#x0930;&#x093E;&#x0920;&#x0940;';
  ja = '&#x65E5;&#x672C;&#x8A9E;';
  ta = '&#x0BA4;&#x0BAE;&#x0BBF;&#x0BB4;&#x0BCD;';
  te = '&#x0C24;&#x0C46;&#x0C32;&#x0C41;&#x0C17;&#x0C41;';
  ur = '&#x0627;&#x0631;&#x062F;&#x0648;';
  zh = '&#x4E2D;&#x6587;';

function Decode(S: string): WideString;
var
  I: Integer;
  Sub: string;
begin
  for I := 0 to Trunc(Length(S) / 8) - 1 do
  begin
    Sub := Copy(S, 8 * i + 4, 4);
    Result := Result + WideChar(StrToInt('$' + Sub));
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  MemoLog := TTntMemo.Create(Self);
  MemoLog.Parent := Panel1;
  MemoLog.Align := alClient;

  TntComboBox1 := TTntComboBox.Create(Self);
  TntComboBox1.Parent := Panel1;
  TntComboBox1.Left := 88;
  TntComboBox1.Top := 31;
  TntComboBox1.Width := 390;
  TntComboBox1.Anchors := [akLeft, akTop, akRight];

  MemoIncoming := TTntMemo.Create(Self);
  MemoIncoming.Parent := Panel3;
  MemoIncoming.Align := alClient;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  MemoLog.Clear;
  MemoIncoming.Clear;

  TntComboBox1.Items.Add(Decode(am));
  TntComboBox1.Items.Add(Decode(ar));
  TntComboBox1.Items.Add(Decode(gu));
  TntComboBox1.Items.Add(Decode(he));
  TntComboBox1.Items.Add(Decode(hi));
  TntComboBox1.Items.Add(Decode(kn));
  TntComboBox1.Items.Add(Decode(mr));
  TntComboBox1.Items.Add(Decode(ja));
  TntComboBox1.Items.Add(Decode(ta));
  TntComboBox1.Items.Add(Decode(te));
  TntComboBox1.Items.Add(Decode(ur));
  TntComboBox1.Items.Add(Decode(zh));

  TntComboBox1.ItemIndex := 4;

  Caption := Format('Habari ActiveMQ Unicode demo (RtlVersion: %f)',
    [RtlVersion]);
end;

procedure TMainForm.Log(const Msg: WideString);
begin
  MemoLog.Lines.Add(Msg);
end;

procedure TMainForm.ButtonSendClick(Sender: TObject);
var
  Connection: IConnection;
  Session: ISession;
  Destination: IDestination;
  Producer: IMessageProducer;
begin
  Connection := TBTJMSConnection.MakeConnection;
  Connection.Start;
  try
    Session := Connection.CreateSession(False, amAutoAcknowledge);
    Destination := Session.CreateQueue('unicodetest');
    Producer := Session.CreateProducer(Destination);
    Producer.Send(Session.CreateTextMessage(TntComboBox1.Text));
    Log('Send: ' + TntComboBox1.Text);
  finally
    Connection.Close;
  end;
end;

procedure TMainForm.ButtonReceiveClick(Sender: TObject);
var
  Connection: IConnection;
  Session: ISession;
  Destination: IDestination;
  Consumer: IMessageConsumer;
  Msg: IMessage;
  MsgText: WideString;
begin
  Connection := TBTJMSConnection.MakeConnection;
  try
    Connection.Start;
    try
      Session := Connection.CreateSession(False, amClientAcknowledge);
      Destination := Session.CreateQueue('unicodetest');
      Consumer := Session.CreateConsumer(Destination);
      Msg := Consumer.Receive;
      if Assigned(Msg) then
      begin
        if Supports(Msg, ITextMessage) then
        begin
          MsgText := (Msg as ITextMessage).Text;
          MemoIncoming.Lines.Add('Received: ' + MsgText);
          Msg.Acknowledge;
        end;
      end
      else
      begin
        MemoIncoming.Lines.Add('No message received')
      end;
    finally
      Connection.Close;
    end;
  except
    on E: Exception do
    begin
      ShowMessage(E.Message);
    end
  end;
end;

end.

